from itertools import chain
from typing import List

from astropy.io import fits
from edps import RecipeInvocationArguments, InvokerProvider, ProductRenamer, RecipeInvocationResult
from edps import subworkflow, qc0, qc1calib, task, File, ReportInput

from .matisse_task_functions import run_recipe


def _loop_merge_recipe(input_files: List[File], indexes: List[int], args: RecipeInvocationArguments,
                       invoker_provider: InvokerProvider, renamer: ProductRenamer) -> List[RecipeInvocationResult]:
    results = []
    for index in indexes:
        parameters = args.parameters.copy()
        parameters['matisse.mat_merge_results.bcdMode'] = str(index)
        result = run_recipe(input_files, [], parameters, 'mat_merge_results', args, invoker_provider.recipe_invoker,
                            renamer)
        results.append(result)
    return results


def loop_merge_recipe(args: RecipeInvocationArguments, invoker_provider: InvokerProvider,
                      renamer: ProductRenamer) -> RecipeInvocationResult:
    # This function execute the recipe mat_merge_results for times, using different values for the parameter bcdMode.
    # Results of each execution are appended together and returned to the main task.
    science_files = [File(f.name, f.category, "") for f in args.inputs.combined]
    science_files_chop = []
    science_files_nochop = []
    results = []

    # If there are no inputs then an empty list is returned with return code = 1
    if len(science_files) == 0:
        return RecipeInvocationResult(return_code=1, output_files=[])

    # Separating observations done in chopping and no-chopping modes
    for f in science_files:
        with fits.open(f.file_path) as hdul:
            chop = hdul[0].header.get("HIERARCH ESO ISS CHOP ST")
            if chop is True or chop == 'T':
                science_files_chop.append(f)
            else:
                science_files_nochop.append(f)

    # Processing observations with  ISS.CHOP.ST = TRUE
    if science_files_chop:
        results.extend(_loop_merge_recipe(science_files_chop, [4, 5], args, invoker_provider, renamer))
    # Processing observations with  ISS.CHOP.ST = FALSE (or missing)
    if science_files_nochop:
        results.extend(_loop_merge_recipe(science_files_nochop, [0, 1, 2, 3], args, invoker_provider, renamer))

    failed_executions = [res for res in results if res.return_code != 0]
    if failed_executions:
        return RecipeInvocationResult(return_code=1, output_files=[])
    else:
        output_files = chain.from_iterable([res.output_files for res in results])
        return RecipeInvocationResult(return_code=0, output_files=list(output_files))


@subworkflow("merge_results", "")
def merge_results(incoherent, coherent, data_type, transfer_function=None):
    # Subworkflow to merge the uncalibrated visibilities.
    #
    # The recipe mat_merge_results needs to be executed four times with different value of the parameter that selects
    # one of the four configurations of the beam commuting devices (BCD). Therefore, the merge tasks needs to run
    # a function (loop_merge_recipe) that loops on the mat_merge_results recipe using different recipe parameters.
    #
    # The input data_type ("calibrator"/"science") determines the task name. For "calibrator" data types, the transfer
    # function can to be provided.

    merge = (task("merge_" + data_type)
             .with_function(loop_merge_recipe)
             .with_main_input(incoherent)
             .with_associated_input(coherent, min_ret=0))

    if transfer_function is not None:
        merge = merge.with_associated_input(transfer_function, min_ret=0)

    if data_type == "science":
        merge = merge.with_meta_targets([qc0, "merge"])
    elif data_type in ["calibrator", "internal_lamp"]:
        merge = (merge
                 .with_report("matisse_calibrator", ReportInput.RECIPE_INPUTS_OUTPUTS)
                 .with_meta_targets([qc1calib, "merge"]))

    return merge.build()
